use std::{
    sync::{Arc, Mutex},
    thread,
    time::{Duration, SystemTime, UNIX_EPOCH},
};

use crate::db::DataBase;

mod db;
mod devices;
mod tasmota;

use anyhow::Result;
use devices::Devices;
use futures::{future::try_join_all, try_join};
use tasmota::Tasmota;

fn since_epoch() -> Result<u64> {
    Ok(SystemTime::now().duration_since(UNIX_EPOCH)?.as_secs())
}

#[tokio::main]
async fn main() -> Result<()> {
    let db_future = DataBase::new("home_server.db");
    let devices_future = Devices::read("devices.conf");

    let (db, devices) = try_join!(db_future, devices_future)?;

    let shared_db = Arc::new(Mutex::new(db));
    let tasmota_plugs: Vec<Tasmota> = devices
        .plugs
        .iter()
        .map(|plug| Tasmota::new(plug))
        .collect();

    loop {
        try_join_all(tasmota_plugs.iter().map(|plug| async {
            if let Ok(usage) = plug.read_power_usage().await {
                shared_db
                    .lock()
                    .unwrap()
                    .write(plug.name(), since_epoch()?, usage)?;
            }

            Ok::<(), anyhow::Error>(())
        }))
        .await?;

        thread::sleep(Duration::from_secs(3));
    }
}
